
----------------------------------------------------------------------------------------------------------------------------------
-- QuickTalents
----------------------------------------------------------------------------------------------------------------------------------
CreateFrame("Frame","QuickTalents",UIParent):RegisterEvent("ADDON_LOADED");
QuickTalents:SetScript("OnEvent",function(self)
	
	-- Settings
	local settings = { -- defaults
		ButtonSize = {"number",28,"Size of the Talent Icons."},
		ShowReagents = {"boolean",true,"Whether or not to display Reagent count."},
		ShowTooltips = {"boolean",false,"Whether or not to show tooltips for talents."},
		BackgroundAlpha = {"number",0.5,"Visibility of the frame background."},
		Position = {"table",{"TOPRIGHT",-5,-210}},
		Bindings = {"table",{}}, -- TODO: maybe make this character specific.
	};
	
	-- load user settings
	QT_Saved = QT_Saved or {};
	for k,v in pairs(settings) do
		if QT_Saved[k] == nil or type(QT_Saved[k]) ~= v[1] then
			QT_Saved[k] = v[2];
		end
	end
	local s = QT_Saved;
	
	-- Learn Queue
	local Queue = {};
	function self:Learn(t)
		PlayerTalentFrame_ClearTalentSelections();
		local r = ceil(t/3);
		Queue[r] = nil;
		for i = r*3-2, r*3 do
			if select(5,GetTalentInfo(i)) then
				Queue[r] = t;
				return
			end
		end
		LearnTalents(t);
	end
	
	-- Event Handler
	self:RegisterEvent("PLAYER_TALENT_UPDATE");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("BAG_UPDATE_DELAYED"); -- for reagents
	self:SetScript("OnEvent",function(self,e)
		if e == "PLAYER_ENTERING_WORLD" then
			self:SetupFrames()
		else
			for i,t in pairs(Queue) do
				self:Learn(t);
			end
		end
		self:UpdateFrames();
	end);
	
	-- Setup main frame
	self:SetSize( s.ButtonSize*3+1, s.ButtonSize*6+(s.ShowReagents and 16 or 1) );
	self:SetPoint(unpack(s.Position));
	self:CreateTexture("QuickTalentsBackground"):SetAllPoints();
	QuickTalentsBackground:SetTexture( 0,0,0, s.BackgroundAlpha );
	-- Make movable
	self:EnableMouse(true);
	self:SetMovable(true);
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnDragStart", self.StartMoving)
	self:SetScript("OnDragStop", function()
		self:StopMovingOrSizing();
		QT_Saved.Position = {self:GetPoint()};
	end)
	
	-- Frame Handler
	local frames = nil;
	function self:SetupFrames()
		frames = {};
		for i = 1,18 do
			local col,row = (i-1)%3,floor((i-1)/3);
			
			frames[i] = CreateFrame("BUTTON","QuickTalentsButton"..i,self,"SecureActionButtonTemplate");
			frames[i]:SetAttribute("type","macro");
			frames[i]:SetAttribute("macrotext",
				"/click QuickTalentsOpener\n".. -- ensures the talent frame is ready for interaction
				"/click PlayerTalentFrameTalentsTalentRow"..(row+1).."Talent"..(col+1).."\n".. -- only way(?) to get the unlearn popup without taint
				"/click StaticPopup1Button1\n".. -- confirm unlearn (TODO: what if popup1 is not the talent prompt)
				"/run QuickTalents:Learn("..i..")" -- queue new talents for learn
			);
			frames[i]:SetSize(s.ButtonSize-3,s.ButtonSize-3);
			frames[i]:SetPoint("TOPLEFT",col*s.ButtonSize+2,-(row*s.ButtonSize+2));
			frames[i]:RegisterForDrag("LeftButton");
			frames[i]:SetScript("OnLeave",function() GameTooltip:Hide(); end);
			
			-- texture
			frames[i].t = frames[i]:CreateTexture("QuickTalentsButton"..i.."Texture");
			frames[i].t:SetAllPoints();
			frames[i].t:SetTexture((select(2,GetTalentInfo(i))));
			frames[i].t:SetTexCoord(0.075,0.925,0.075,0.925);
			
			-- tooltip & dragging
			frames[i]:SetScript("OnEnter", function(self)
				if s.ShowTooltips then
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
					GameTooltip:SetTalent(i);
					GameTooltip:Show();
				end
			end);
			frames[i]:SetScript("OnDragStart",function()
				if not InCombatLockdown() then
					PickupTalent(i)
					if CursorHasSpell() then
						QuickTalentsBinder.spell = select(4,GetCursorInfo());
						QuickTalentsBinder:SetScript("OnUpdate",QuickTalentsBinder.OnEvent);
					end
				end
			end);
		end
	end
	function self:UpdateFrames()
		if not frames then return end;
		for i = 1,18 do
			frames[i].t:SetVertexColor(1,1,1,select(5,GetTalentInfo(i)) and 1 or 0.25);
			SetDesaturation(frames[i].t,InCombatLockdown());
		end
		-- update reagent count
		if s.ShowReagents then
			QuickTalentsReagents:SetText(select(2,GetTalentClearInfo()));
		end
	end
	
	-- Handles the safe loading & opening of the Blizzard Talent UI
	-- Opening the talent ui triggers a setup sequence which has to be done securely to avoid tainting the whole system.
	LoadAddOn("Blizzard_TalentUI");
	CreateFrame("BUTTON","QuickTalentsOpener",self,"SecureActionButtonTemplate"):SetAttribute("type","macro");
	local UpdateOpener = function()
		QuickTalentsOpener:SetAttribute("macrotext",
			"/click TalentMicroButton\n"..
			"/click PlayerTalentFrameTab2\n"..
			"/click PlayerSpecTab"..GetActiveSpecGroup().."\n"..
			"/click TalentMicroButton"
		);
	end
	QuickTalentsOpener:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	QuickTalentsOpener:SetScript("OnEvent",UpdateOpener);
	UpdateOpener();
	
	-- Reagent/clear count
	self:CreateFontString("QuickTalentsReagents"):SetFont("Fonts\\ARIALN.TTF", 13, "OUTLINE");
	QuickTalentsReagents:SetPoint("BOTTOMLEFT",2,2.5);
	
	-- Binder
	CreateFrame("Frame","QuickTalentsBinder",self);
	function QuickTalentsBinder:OnEvent(e,arg1)
		if e == "LEARNED_SPELL_IN_TAB" then
			if s.Bindings[arg1] then
				PickupSpell(arg1);
				PlaceAction(s.Bindings[arg1]);
				ClearCursor();
			end
		else
			if self.spell and select(4,GetCursorInfo()) ~= self.spell then
				if e == "ACTIONBAR_SLOT_CHANGED" then
					PlaySound("TellMessage");
					print(format("|HQT:bind:%d:%d|h[Click Here]|h",self.spell,arg1).." if you want to bind "..GetSpellLink(self.spell).." to action slot "..arg1..".");
				end
				self.spell = nil;
				self:SetScript("OnUpdate",nil);
			end
		end
	end
	QuickTalentsBinder:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	QuickTalentsBinder:RegisterEvent("LEARNED_SPELL_IN_TAB");
	QuickTalentsBinder:SetScript("OnEvent",QuickTalentsBinder.OnEvent)
	
	-- hook chatframe hyperlink handler for bind links
	local _OHLC = ChatFrame1:GetScript("OnHyperLinkClick");
	ChatFrame1:SetScript("OnHyperLinkClick",function(self,link,test,btn)
		local arg = {strsplit(":",link)};
		if arg[1] == "QT" then
			if arg[2] == "bind" then
				QT_Saved.Bindings[tonumber(arg[3])] = tonumber(arg[4]);
				print(GetSpellLink(arg[3]).." bound to action slot "..arg[4]..".");
			end
			return;
		end
		_OHLC(self,link,text,btn);
	end);
	
	-- Settings Editor
	SLASH_QUICKTALENTS1 = "/quicktalents";
	SlashCmdList.QUICKTALENTS = function(args)
		local arg1,arg2 = strsplit(" ",args);
		if settings[arg1] then
			if settings[arg1][1] == "boolean" then
				QT_Saved[arg1] = strlower(arg2) == "true";
			elseif settings[arg1][1] == "number" then
				QT_Saved[arg1] = tonumber(arg2);
			end
			self:SettingsChanged();
			print(arg1..": [|cFF3399FF"..tostring(QT_Saved[arg1]).."|r]");
		else
			print("|cFF009999QuickTalents Settings:|r");
			for k,v in pairs(settings) do
				if v[3] then
					print(k..": [|cFF3399FF"..tostring(QT_Saved[k]).."|r] - |cFFAAAAAA"..v[3].."|r");
				end
			end
		end
	end
	
	-- Updates frames after setting changes
	function self:SettingsChanged()
		self:SetSize( s.ButtonSize*3+1, s.ButtonSize*6+(s.ShowReagents and 16 or 1) );
		for i = 1,18 do
			frames[i]:SetSize(s.ButtonSize-3,s.ButtonSize-3);
			frames[i]:SetPoint("TOPLEFT",(i-1)%3*s.ButtonSize+2,-(floor((i-1)/3)*s.ButtonSize+2));
		end
		QuickTalentsBackground:SetTexture( 0,0,0, s.BackgroundAlpha );
		QuickTalentsReagents:SetText("");
		self:UpdateFrames();
	end
	
end);